<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 03/22/2004 *
 * ===================                          Last change: 12/13/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : what-extentions.php                              *
 * -------------------------------------------------------------------- *
 * Short description : Extension management                             *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Erweiterungen-Management                         *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2015 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if ((!defined('__SECURITY')) || (!isAdmin())) {
	die();
} // END - if

// Add description as navigation point
addYouAreHereLink('admin', __FILE__);

// By default generate overview of all installed extensions
$do = 'overview';

if (isGetRequestElementSet('register_ext')) {
	// We are about to register a new extension
	$do = 'setup';
	$taskId = determineExtensionTaskId(getRequestElement('register_ext'));

	// The id comes from task management and it is - of course - *not* the extension's name!
} elseif ((isFormSent('change')) && (ifPostContainsSelections()) && (!isDemoModeActive())) {
	// De-/activate extensions
	foreach (postRequestElement('sel') as $ext_id => $ext_active) {
		// Shall we keep the extension always active?
		if ((isset($GLOBALS['cache_array']['always_active'][getExtensionName($ext_id)])) && ($GLOBALS['cache_array']['always_active'][getExtensionName($ext_id)] == 'Y') && ($ext_active == 'Y')) {
			// Keep this extension active!
			displayMessage('{%message,ADMIN_EXTENSION_ALWAYS_ACTIVE=' . $ext_id . '%}');
		} else {
			// Deactivation is default
			$active = 'N';
			setExtensionMode('deactivate');

			// Is the extension deactivated?
			if ($ext_active != 'Y') {
				// Then we can activate it
				$active = 'Y';
				setExtensionMode('activate');
			} // END - if

			// Update database
			sqlQueryEscaped("UPDATE `{?_MYSQL_PREFIX?}_extensions` SET `ext_active`='%s' WHERE `id`=%s AND `ext_active`='%s' LIMIT 1",
				array(
					$active,
					bigintval($ext_id),
					$ext_active
				), __FILE__, __LINE__);

			// Run embeded SQL commands
			doExtensionSqls($ext_id, getExtensionMode());
		}
	} // END - foreach

	// Redirect to prevent missing cache in js.php
	redirectToUrl('modules.php?module=admin&amp;what=extensions&amp;changed=' . countPostSelection());
} elseif (((isFormSent('edit')) || (isPostRequestElementSet('do_edit'))) && (ifPostContainsSelections()) && (!isDemoModeActive())) {
	// Change settings like CSS file load
	if (isPostRequestElementSet('do_edit')) {
		// Change entries
		$cache_update = '0';
		foreach (postRequestElement('sel') as $ext_id => $sel) {
			// Secure id
			$ext_id = bigintval($ext_id);

			// Change this extension?
			if ($sel == 1) {
				// Load mode is modify
				setExtensionMode('modify');

				// Get entry for 'active'
				$active = postRequestElement('active', $ext_id);

				// Update extension's record
				if (isExtensionInstalledAndNewer('sql_patches', '0.0.6')) {
					// Update also CSS column when extensions ext-sql_patches is newer or exact v0.0.6
					sqlQueryEscaped("UPDATE `{?_MYSQL_PREFIX?}_extensions` SET `ext_has_css`='%s',`ext_active`='%s' WHERE `id`=%s LIMIT 1",
						array(postRequestElement('css', $ext_id), $active, $ext_id), __FILE__, __LINE__);
				} else {
					// When extension is older than v0.0.6 there is no column for the CSS information
					sqlQueryEscaped("UPDATE `{?_MYSQL_PREFIX?}_extensions` SET `ext_active`='%s' WHERE `id`=%s LIMIT 1",
						array($active, $ext_id), __FILE__, __LINE__);
				}

				// Run SQLs on activation / deactivation
				switch ($active) {
					case 'Y': setExtensionMode('activate');   break;
					case 'N': setExtensionMode('deactivate'); break;
				} // END - switch

				// Run embeded SQL commands
				doExtensionSqls($ext_id, getExtensionMode());
			} // END - if
		} // END - foreach

		// Redirect to prevent missing cache in js.php
		redirectToUrl('modules.php?module=admin&amp;what=extensions&amp;edited=' . countPostSelection());
	} else {
		// Edit selected entries
		$OUT = '';
		foreach (postRequestElement('sel') as $ext_id => $sel) {
			// Edit this extension?
			if (($sel == 'Y') || ($sel != 'Y')) {
				// Default is no CSS for non-updated mailers
				$css = 'N';

				// Load required data
				if (isExtensionInstalledAndNewer('sql_patches', '0.0.6')) {
					$result = sqlQueryEscaped("SELECT `ext_has_css`, `ext_active` FROM `{?_MYSQL_PREFIX?}_extensions` WHERE `id`=%s LIMIT 1",
						array(bigintval($ext_id)), __FILE__, __LINE__);
					list($css, $active) = sqlFetchRow($result);
					sqlFreeResult($result);
				} else {
					$result = sqlQueryEscaped("SELECT `ext_active` FROM `{?_MYSQL_PREFIX?}_extensions` WHERE `id`=%s LIMIT 1",
						array(bigintval($ext_id)), __FILE__, __LINE__);
					list($active) = sqlFetchRow($result);
					sqlFreeResult($result);
					$css = 'X';
				}

				// Prepare CSS column
				$cssSelection = '---';
				if (isExtensionInstalledAndNewer('sql_patches', '0.0.6')) {
					$cssSelection = addSelectionBox('yn', $css, 'css', $ext_id);
				} // END - if

				// Prepare data for the row template
				$content = array(
					'id'     => $ext_id,
					'name'   => getExtensionName($ext_id),
					'active' => addSelectionBox('yn', $active, 'active', $ext_id),
					'css'    => $cssSelection,
				);

				// Load row template and switch color
				$OUT .= loadTemplate('admin_edit_extensions_row', TRUE, $content);
			} // END - if
		} // END - foreach

		// Load template
		loadTemplate('admin_edit_extensions', FALSE, $OUT);
		$do = 'edit';
	}
} elseif ((isPostRequestElementSet('delete')) && (ifPostContainsSelections()) && (!isDemoModeActive())) {
	// List extensions and when verbose is enabled SQL statements which will be executed
	$OUT = '';
	foreach (postRequestElement('sel') as $ext_id => $sel) {
		// Init variables
		$VERBOSE_OUT = '';
		initSqls();

		// Secure id number
		$ext_id = bigintval($ext_id);

		// Is the id valid?
		if (isExtensionIdValid($ext_id)) {
			// Get extension name
			$ext_name = getExtensionName($ext_id);

			// Listing of SQLs enabled?
			if (isVerboseSqlEnabled()) {
				// Load SQL commands in remove mode
				if (loadExtension($ext_name, 'remove', '0.0.0', TRUE)) {
					// Generate extra table with loaded SQL commands
					$VERBOSE_OUT = addExtensionVerboseSqlTable();
				} // END - if
			} // END - if

			// Prepare data for the row template
			$content = array(
				'id'       => $ext_id,
				'ext_name' => $ext_name,
				'verbose'  => $VERBOSE_OUT
			);

			// Load row template and switch color
			$OUT .= loadTemplate('admin_delete_extensions_row', TRUE, $content);
		} else {
			// Prepare data for the row template
			$content = array(
				'id'       => $ext_id
			);

			// Not valid!
			$OUT .= loadTemplate('admin_delete_extensions_row_404', TRUE, $content);
		}
	} // END - foreach

	// Load template
	loadTemplate('admin_delete_extensions', FALSE, $OUT);
	$do = 'delete';
} elseif ((isFormSent('do_delete')) && (ifPostContainsSelections()) && (!isDemoModeActive())) {
	// Remove extensions from DB (you have to delete all files manually!)
	$cache_update = '0';
	foreach (postRequestElement('sel') as $ext_id => $active) {
		// Secure id number
		$ext_id = bigintval($ext_id);

		// Is this extension selected and id valid?
		if (($active == 1) && (isExtensionIdValid($ext_id))) {
			// Run embeded SQL commands
			doExtensionSqls($ext_id, 'remove');
		} // END - if
	} // END - foreach

	// Redirect to prevent missing cache in js.php
	redirectToUrl('modules.php?module=admin&amp;what=extensions&amp;removed=' . countPostSelection());
} elseif ((isGetRequestElementSet('do')) && (!isDemoModeActive())) {
	// Other things to do
	$do = getRequestElement('do');
} elseif (isGetRequestElementSet('do')) {
	// Demo mode active!
	$do = 'demo';
} elseif (isGetRequestElementSet('registered')) {
	// Extensions changed
	displayMessage('{%message,ADMIN_EXTENSION_REGISTERED=' . getExtensionName(getRequestElement('registered')) . '%}');

	// Show next link
	if (isExtensionActive('task')) {
		loadTemplate('admin_next_link', FALSE, array(
			'url'   => 'modules.php?module=admin&amp;what=list_task',
			'title' => '{--ADMIN_EXTENSION_REGISTER_NEXT_LINK--}'
		));
	} else {
		loadTemplate('admin_next_link', FALSE, array(
			'url'   => 'modules.php?module=admin',
			'title' => '{--ADMIN_EXTENSION_REGISTER_NEXT_LINK--}'
		));
	}
} elseif (isGetRequestElementSet('changed')) {
	// Extensions changed
	displayMessage('{%message,ADMIN_EXTENSION_CHANGED=' . bigintval(getRequestElement('changed')) . '%}');
} elseif (isGetRequestElementSet('edited')) {
	// Extensions changed
	displayMessage('{%message,ADMIN_EXTENSION_EDITED=' . bigintval(getRequestElement('edited')) . '%}');
} elseif (isGetRequestElementSet('removed')) {
	// Extensions changed
	displayMessage('{%message,ADMIN_EXTENSIONS_REMOVED=' . bigintval(getRequestElement('removed')) . '%}');
}

// Shall we display active/inactive extensions?
$where = '';
if (isGetRequestElementSet('active')) {
	$where = sprintf("WHERE `ext_active`='%s'", sqlEscapeString(getRequestElement('active')));
} // END - if

// Case selection
switch ($do) {
	case 'overview': // List all registered extensions
		if (isExtensionInstalledAndNewer('sql_patches', '0.0.6')) {
			// Load extension data with CSS informations
			$result = sqlQuery('SELECT
	`id` AS `ext_id`,
	`ext_name`,
	`ext_active`,
	`ext_has_css` AS `ext_css`,
	`ext_version`
FROM
	`{?_MYSQL_PREFIX?}_extensions`
' . $where . '
ORDER BY
	`ext_name` ASC', __FILE__, __LINE__);
		} else {
			// Load extension data without CSS informations
			$result = sqlQuery('SELECT
	`id` AS `ext_id`,
	`ext_name`,
	`ext_active`,
	`ext_version`
FROM
	`{?_MYSQL_PREFIX?}_extensions`
' . $where . '
ORDER BY
	`ext_name` ASC', __FILE__, __LINE__);
		}

		// Are there some entries?
		if (!ifSqlHasZeroNums($result)) {
			// Extensions are registered
			$OUT = '';
			while ($content = sqlFetchArray($result)) {
				// Prepare CSS selection output
				$cssSelection = '---';
				if (isExtensionInstalledAndNewer('sql_patches', '0.0.6')) {
					$cssSelection = '{%pipe,translateYesNo=' . $content['ext_css'] . '%}';
				} // END - if

				// Prepare data for the row template
				$content['ext_css'] = $cssSelection;

				// Load row template and switch color
				$OUT .= loadTemplate('admin_list_extensions_row', TRUE, $content);
			} // END - while

			// Free memory
			sqlFreeResult($result);

			// Load template
			loadTemplate('admin_list_extensions', FALSE, $OUT);
		} else {
			// No extensions are registered
			displayMessage('{--ADMIN_NO_EXTENSION_REGISTERED--}');
		}
		break;

	case 'setup': // Setup new extension
		// Are there some tasks?
		$numTasks = countSumTotalData(getCurrentAdminId(), 'task_system', 'id', 'assigned_admin', TRUE, "AND `task_type`='EXTENSION'");

		// Is the id number valid and the task was found?
		if ((isValidId($taskId)) && ($numTasks > 0)) {
			// id is valid so begin with registration, we first want to it's real name from task management (subject column)
			$result = sqlQueryEscaped("SELECT `subject` FROM `{?_MYSQL_PREFIX?}_task_system` WHERE `id`=%s LIMIT 1",
				array(bigintval($taskId)), __FILE__, __LINE__);
			list($subject) = sqlFetchRow($result);

			// Free result
			sqlFreeResult($result);

			// Disable cache update by default
			$cache_update = '0';
			if (!empty($subject)) {
				// Extract extension's name from subject...
				$ext_name = trim(substr($subject, 1, strpos($subject, ':') - 1));

				// Test the extension for deprecation
				loadExtension($ext_name, 'test');

				// Is the extension deprecated?
				if (!isExtensionDeprecated()) {
					// ... so we can finally register and load it in registration mode
					if (registerExtension($ext_name, $taskId)) {
						// Do we need to update cache file?
						// @TODO Rewrite this to a filter
						if (isExtensionActive('cache')) {
							// Remove cache file (will be auto-created again!)
							foreach (array('config', 'extension', 'filter', 'modules') as $cache) {
								// Rebuild cache file
								rebuildCache($cache);
							} // END - foreach
						} // END - if

						// Errors?
						if (!ifFatalErrorsDetected()) {
							// Extension was found and successfully registered
							redirectToUrl('modules.php?module=admin&amp;what=extensions&amp;registered=' . getExtensionId($ext_name));
						} else {
							// Errors detected!
							displayMessage('{%message,ADMIN_EXTENSION_NOT_REGISTERED=' . $ext_name . '%}');
						}
					} elseif (isExtensionInstalled($ext_name)) {
						// Notify the admin that we have a problem here...
						displayMessage('{--ADMIN_EXTENSION_REGISTRATION_FAILED_ALREADY--}');
					} else {
						// Notify the admin that we have a problem here...
						displayMessage('{%message,ADMIN_EXTENSION_REGISTRATION_FAILED_404=' . $ext_name . '%}');
					}
				} else {
					// Motify the admin that we have a problem here...
					displayMessage('{--ADMIN_EXTENSION_REGISTRATION_FAILED_DEPRECATED--}');
				}
			} else {
				// Extension was not found in task management
				displayMessage('{--ADMIN_EXTENSION_REGISTRATION_FAILED_ID_404--}');
			}
		} elseif ($numTasks == '0') {
			// No longer assigned or old task
			displayMessage('{--ADMIN_EXTENSION_REGISTRATION_FAILED_ASSIGED--}');
		} else {
			// id is invalid
			displayMessage('{--ADMIN_EXTENSION_REGISTRATION_FAILED_INVALID--}');
		}
		break;

	case 'demo':
		displayMessage('{--SETTINGS_NOT_SAVED--}');
		break;
} // END - switch

// [EOF]
?>
