<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 10/06/2012 *
 * ===================                          Last change: 10/06/2012 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : what-subids.php                                  *
 * -------------------------------------------------------------------- *
 * Short description : Lists and manages member's own sub ids           *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Auflisten und verwalten der Mitglied-SubIds      *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2015 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	exit();
} elseif (!isMember()) {
	redirectToIndexMemberOnlyModule();
}

// Add description as navigation point
addYouAreHereLink('member', __FILE__);

// Continue only if the proper extension is active (admins can always continue)
if ((!isExtensionActive('user')) && (!isAdmin())) {
	// Required extension ext-user not active
	displayMessage('{%pipe,generateExtensionInactiveNotInstalledMessage=user%}');

	// Abort here
	return;
} // END - if

// By default show the list...
$show = TRUE;

// Check for 'url_id' element
//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'isFormSent()=' . intval(isFormSent('add_subid')) . ',ifPostContainsSelections()=' . intval(ifPostContainsSelections()) . ',countRequestPost()=' . countRequestPost());
if ((countRequestPost('id') > 0) && (!isFormSent('add_subid')) && ((!ifPostContainsSelections('id')) || (countRequestPost('id') == 0))) {
	// Not found so output message
	displayMessage('{--MEMBER_SUBID_NO_SELECTIONS--}');

	// Abort here
	return;
} // END - if

// Edit or delete button hit?
if (isFormSent('add_subid')) {
	// Add new sub id
	doGenericXmlTemplateCallback('member_add_do_user_subid');
} elseif (isFormSent('edit')) {
	// Show entries for editing
	doGenericXmlTemplateCallback('member_edit_show_user_subid');

	// Do not show the list of URLs after this template
	$show = FALSE;
} elseif (isFormSent('do_edit')) {
	// Change data of entries
	doGenericXmlTemplateCallback('member_edit_do_user_subid');
} elseif (isFormSent('delete')) {
	// Show entries for deletion
	doGenericXmlTemplateCallback('member_delete_show_user_subid');

	// Do not show the list of URLs after this template
	$show = FALSE;
} elseif (isFormSent('do_delete')) {
	// Remove entries from database
	doGenericXmlTemplateCallback('member_delete_do_user_subid');
} elseif ((isGetRequestElementSet('do')) && (isGetRequestElementSet('id'))) {
	// Construct module name
	$incFile = sprintf('inc/modules/member/subid-%s.php', sqlEscapeString(getRequestElement('do')));

	// Is the include readable and is the sub id assigned to current member?
	if (!isUserSubIdAssignedToMember(getRequestElement('id'))) {
		// Sub id is not assigned to current member or doesn't exist
		displayMessage('{--MEMBER_USER_SUBID_NOT_ASSIGNED_404--}');
	} elseif (isIncludeReadable($incFile)) {
		// Then include it
		loadIncludeOnce($incFile);

		// Do not show the list of URLs after this template
		$show = FALSE;
	} else {
		// Display error message
		displayMessage('{--MEMBER_INVALID_DO_USER_SUBID--}');
	}
}

// Show entries?
if ($show === FALSE) {
	// No, a form has already been show
	return FALSE;
} // END - if

// List all campaigns
doGenericXmlTemplateCallback('member_list_subid');

// Display form for adding a sub id
loadTemplate('member_add_subid');

// [EOF]
?>
