/**
 * JavaScript for admin ext-network AJAX stuff
 * --------------------------------------------------------------------
 * Copyright (c) 2003 - 2009 by Roland Haeder
 * Copyright (c) 2009 - 2015 by Mailer Developer Team
 * For more information visit: http://mxchange.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */

// Sends a request out to query a network API
function sendNetworkQueryByTypeId (typeId) {
	// Send out the request
	return sendAjaxRequest('admin', 'network_query_single_api', '&network_type_id=' + typeId, false);
}

// Queries a single network type handler
function doQueryNetworkType (typeId, displayMessage) {
	// Close all windows
	closeAllWindows('admin');

	// Update status
	updateStatusField('status_' + typeId, '', 'CHECKING ...');

	// Does the request work?
	if (sendNetworkQueryByTypeId(typeId) === false) {
		// Display a message?
		if (displayMessage === true) {
			// Display error message
			displayErrorWindow('admin', getAjaxContent());
		} // END - if
	} else {
		// Display a message?
		if (displayMessage === true) {
			// Display success message
			displaySuccessWindow('admin', getAjaxContent());
		} // END - if

		// Update status
		updateStatusField('status_' + typeId, 'good', 'OKAY');

		// Worked fine
		return true;
	}

	// Update status
	updateStatusField('status_' + typeId, 'bad', 'ERROR!');

	// Didn't work
	return false;
}

// Query all network APIs
// @TODO Add support for checkboxes
// @TODO Make progress window working
function doQueryNetwork (networkId) {
	// Open progress window
	//displayProgressWindow('admin', 'Init ...');

	// Do the AJAX request (JSON as content is enabled)
	if (sendAjaxRequest('admin', 'network_list_by_id', '&network_id=' + networkId, true) === true) {
		// Get content (as it is an array)
		var queries = getAjaxContent();
		var total = getTotalCountFromObject(queries);

		// Parse the content
		$.each(queries, function (i, typeId) {
			// Set progress content
			//setProgressContent('admin', typeId);

			// Request it
			if (doQueryNetworkType(typeId, false) === false) {
				// Stop processing and display error message
				displayErrorWindow('admin', getAjaxContent() + 'typeId=' + typeId);

				// Abort here
				return false;
			} // END - if

			// Update progress bar
			updateProgressBar(queries.length);

			// Wait a little
			$('body').delay(500);
		});

		// Is all completed?
		if (counterSuccess == total) {
			// Display success message
			displaySuccessWindow('admin', 'OKAY');
		} // END - if
	} else {
		// Display error window
		displayErrorWindow('admin', getAjaxContent());
	}
}
